@@ -734,93 +734,115 @@ SCR_TileClear(void)
 		return; /* full screen cinematic */
 	}
 
-	/* erase rect will be the union of the past three
-	   frames so tripple buffering works properly */
-	clear = scr_dirty;
 
-	for (i = 0; i < 2; i++)
-	{
-		if (scr_old_dirty[i].x1 < clear.x1)
+	/* This highly complicated pseudo damage tracking is very effective
+	   with the soft render, it gives about 10% speedup. On the GL
+	   renderers the speedup is negligible, but in introduce a bug:
+	   OpenGL requires to redraw the borders after every glClear(),
+	   which the damage tracking doesn't take into account. Fix this
+	   by not using the damage tracking when running somethinge else
+	   than the soft renderer. Just redraw the borders every frame. */
+	if (strcmp(vid_renderer->string, "soft") != 0) {
+		// Top
+		Draw_TileClear(scr_vrect.x, 0, scr_vrect.width, scr_vrect.y, "backtile");
+
+		// Bottom
+		Draw_TileClear(scr_vrect.x, scr_vrect.y + scr_vrect.height, scr_vrect.width, viddef.height, "backtile");
+
+		// Left
+		Draw_TileClear(0, 0, scr_vrect.x, viddef.height, "backtile");
+
+		// Right
+		Draw_TileClear(scr_vrect.x + scr_vrect.width, 0, viddef.width, viddef.height, "backtile");
+	} else {
+		/* erase rect will be the union of the past three
+		   frames so tripple buffering works properly */
+		clear = scr_dirty;
+
+		for (i = 0; i < 2; i++)
 		{
-			clear.x1 = scr_old_dirty[i].x1;
+			if (scr_old_dirty[i].x1 < clear.x1)
+			{
+				clear.x1 = scr_old_dirty[i].x1;
+			}
+
+			if (scr_old_dirty[i].x2 > clear.x2)
+			{
+				clear.x2 = scr_old_dirty[i].x2;
+			}
+
+			if (scr_old_dirty[i].y1 < clear.y1)
+			{
+				clear.y1 = scr_old_dirty[i].y1;
+			}
+
+			if (scr_old_dirty[i].y2 > clear.y2)
+			{
+				clear.y2 = scr_old_dirty[i].y2;
+			}
 		}
 
-		if (scr_old_dirty[i].x2 > clear.x2)
+		scr_old_dirty[1] = scr_old_dirty[0];
+		scr_old_dirty[0] = scr_dirty;
+
+		scr_dirty.x1 = 9999;
+		scr_dirty.x2 = -9999;
+		scr_dirty.y1 = 9999;
+		scr_dirty.y2 = -9999;
+
+		/* don't bother with anything convered by the console */
+		top = (int)(scr_con_current * viddef.height);
+
+		if (top >= clear.y1)
 		{
-			clear.x2 = scr_old_dirty[i].x2;
+			clear.y1 = top;
 		}
 
-		if (scr_old_dirty[i].y1 < clear.y1)
+		if (clear.y2 <= clear.y1)
 		{
-			clear.y1 = scr_old_dirty[i].y1;
+			return; /* nothing disturbed */
 		}
 
-		if (scr_old_dirty[i].y2 > clear.y2)
+		top = scr_vrect.y;
+		bottom = top + scr_vrect.height - 1;
+		left = scr_vrect.x;
+		right = left + scr_vrect.width - 1;
+
+		if (clear.y1 < top)
 		{
-			clear.y2 = scr_old_dirty[i].y2;
+			/* clear above view screen */
+			i = clear.y2 < top - 1 ? clear.y2 : top - 1;
+			Draw_TileClear(clear.x1, clear.y1,
+					clear.x2 - clear.x1 + 1, i - clear.y1 + 1, "backtile");
+			clear.y1 = top;
 		}
-	}
 
-	scr_old_dirty[1] = scr_old_dirty[0];
-	scr_old_dirty[0] = scr_dirty;
+		if (clear.y2 > bottom)
+		{
+			/* clear below view screen */
+			i = clear.y1 > bottom + 1 ? clear.y1 : bottom + 1;
+			Draw_TileClear(clear.x1, i,
+					clear.x2 - clear.x1 + 1, clear.y2 - i + 1, "backtile");
+			clear.y2 = bottom;
+		}
 
-	scr_dirty.x1 = 9999;
-	scr_dirty.x2 = -9999;
-	scr_dirty.y1 = 9999;
-	scr_dirty.y2 = -9999;
+		if (clear.x1 < left)
+		{
+			/* clear left of view screen */
+			i = clear.x2 < left - 1 ? clear.x2 : left - 1;
+			Draw_TileClear(clear.x1, clear.y1,
+					i - clear.x1 + 1, clear.y2 - clear.y1 + 1, "backtile");
+			clear.x1 = left;
+		}
 
-	/* don't bother with anything convered by the console */
-	top = (int)(scr_con_current * viddef.height);
-
-	if (top >= clear.y1)
-	{
-		clear.y1 = top;
-	}
-
-	if (clear.y2 <= clear.y1)
-	{
-		return; /* nothing disturbed */
-	}
-
-	top = scr_vrect.y;
-	bottom = top + scr_vrect.height - 1;
-	left = scr_vrect.x;
-	right = left + scr_vrect.width - 1;
-
-	if (clear.y1 < top)
-	{
-		/* clear above view screen */
-		i = clear.y2 < top - 1 ? clear.y2 : top - 1;
-		Draw_TileClear(clear.x1, clear.y1,
-				clear.x2 - clear.x1 + 1, i - clear.y1 + 1, "backtile");
-		clear.y1 = top;
-	}
-
-	if (clear.y2 > bottom)
-	{
-		/* clear below view screen */
-		i = clear.y1 > bottom + 1 ? clear.y1 : bottom + 1;
-		Draw_TileClear(clear.x1, i,
-				clear.x2 - clear.x1 + 1, clear.y2 - i + 1, "backtile");
-		clear.y2 = bottom;
-	}
-
-	if (clear.x1 < left)
-	{
-		/* clear left of view screen */
-		i = clear.x2 < left - 1 ? clear.x2 : left - 1;
-		Draw_TileClear(clear.x1, clear.y1,
-				i - clear.x1 + 1, clear.y2 - clear.y1 + 1, "backtile");
-		clear.x1 = left;
-	}
-
-	if (clear.x2 > right)
-	{
-		/* clear left of view screen */
-		i = clear.x1 > right + 1 ? clear.x1 : right + 1;
-		Draw_TileClear(i, clear.y1,
-				clear.x2 - i + 1, clear.y2 - clear.y1 + 1, "backtile");
-		clear.x2 = right;
+		if (clear.x2 > right)
+		{
+			/* clear left of view screen */
+			i = clear.x1 > right + 1 ? clear.x1 : right + 1;
+			Draw_TileClear(i, clear.y1,
+					clear.x2 - i + 1, clear.y2 - clear.y1 + 1, "backtile");
+			clear.x2 = right;
+		}
 	}
 }
 
@@ -1453,6 +1475,8 @@ SCR_Framecounter(void) {
 		char str[10];
 		snprintf(str, sizeof(str), "%3.2ffps", (1000.0 * 1000.0) / (avg / num));
 		DrawStringScaled(viddef.width - scale*(strlen(str)*8 + 2), 0, str, scale);
+		SCR_AddDirtyPoint(viddef.width - scale*(strlen(str)*8 + 2), 0);
+		SCR_AddDirtyPoint(viddef.width, 0);
 	} else if (cl_showfps->value >= 2) {
 		// Calculate average of frames.
 		int avg = 0;
@@ -1484,12 +1508,16 @@ SCR_Framecounter(void) {
 		snprintf(str, sizeof(str), "Min: %7.2ffps, Max: %7.2ffps, Avg: %7.2ffps",
 		         (1000.0 * 1000.0) / min, (1000.0 * 1000.0) / max, (1000.0 * 1000.0) / (avg / num));
 		DrawStringScaled(viddef.width - scale*(strlen(str)*8 + 2), 0, str, scale);
+		SCR_AddDirtyPoint(viddef.width - scale*(strlen(str)*8 + 2), 0);
+		SCR_AddDirtyPoint(viddef.width, 0);
 
 		if (cl_showfps->value > 2)
 		{
 			snprintf(str, sizeof(str), "Max: %5.2fms, Min: %5.2fms, Avg: %5.2fms",
 			         0.001f*min, 0.001f*max, 0.001f*(avg / num));
 			DrawStringScaled(viddef.width - scale*(strlen(str)*8 + 2), scale*10, str, scale);
+			SCR_AddDirtyPoint(viddef.width - scale*(strlen(str)*8 + 2), scale*10);
+			SCR_AddDirtyPoint(viddef.width, scale+10);
 		}
 	}
 }
