@@ -39,9 +39,6 @@ glstate_t gl_state;
 image_t *r_notexture; /* use for bad textures */
 image_t *r_particletexture; /* little dot for particles */
 
-entity_t *currententity;
-model_t *currentmodel;
-
 cplane_t frustum[4];
 
 int r_visframecount; /* bumped when going to a new PVS */
@@ -98,13 +95,18 @@ cvar_t *gl_lightmap;
 cvar_t *gl_shadows;
 cvar_t *gl1_stencilshadow;
 cvar_t *r_mode;
+cvar_t *r_fixsurfsky;
 
 cvar_t *r_customwidth;
 cvar_t *r_customheight;
 
-cvar_t *gl_retexturing;
+cvar_t *r_retexturing;
+cvar_t *r_scale8bittextures;
 
 cvar_t *gl_nolerp_list;
+cvar_t *r_lerp_list;
+cvar_t *r_2D_unfiltered;
+cvar_t *r_videos_unfiltered;
 
 cvar_t *gl1_dynamic;
 cvar_t *r_modulate;
@@ -175,7 +177,7 @@ R_RotateForEntity(entity_t *e)
 }
 
 void
-R_DrawSpriteModel(entity_t *e)
+R_DrawSpriteModel(entity_t *currententity, const model_t *currentmodel)
 {
 	float alpha = 1.0F;
 	vec3_t point[4];
@@ -187,16 +189,16 @@ R_DrawSpriteModel(entity_t *e)
 	   a single polygon without a surface cache */
 	psprite = (dsprite_t *)currentmodel->extradata;
 
-	e->frame %= psprite->numframes;
-	frame = &psprite->frames[e->frame];
+	currententity->frame %= psprite->numframes;
+	frame = &psprite->frames[currententity->frame];
 
 	/* normal sprite */
 	up = vup;
 	right = vright;
 
-	if (e->flags & RF_TRANSLUCENT)
+	if (currententity->flags & RF_TRANSLUCENT)
 	{
-		alpha = e->alpha;
+		alpha = currententity->alpha;
 	}
 
 	if (alpha != 1.0F)
@@ -206,7 +208,7 @@ R_DrawSpriteModel(entity_t *e)
 
 	glColor4f(1, 1, 1, alpha);
 
-	R_Bind(currentmodel->skins[e->frame]->texnum);
+	R_Bind(currentmodel->skins[currententity->frame]->texnum);
 
 	R_TexEnv(GL_MODULATE);
 
@@ -226,16 +228,16 @@ R_DrawSpriteModel(entity_t *e)
 		1, 1
 	};
 
-	VectorMA( e->origin, -frame->origin_y, up, point[0] );
+	VectorMA( currententity->origin, -frame->origin_y, up, point[0] );
 	VectorMA( point[0], -frame->origin_x, right, point[0] );
 
-	VectorMA( e->origin, frame->height - frame->origin_y, up, point[1] );
+	VectorMA( currententity->origin, frame->height - frame->origin_y, up, point[1] );
 	VectorMA( point[1], -frame->origin_x, right, point[1] );
 
-	VectorMA( e->origin, frame->height - frame->origin_y, up, point[2] );
+	VectorMA( currententity->origin, frame->height - frame->origin_y, up, point[2] );
 	VectorMA( point[2], frame->width - frame->origin_x, right, point[2] );
 
-	VectorMA( e->origin, -frame->origin_y, up, point[3] );
+	VectorMA( currententity->origin, -frame->origin_y, up, point[3] );
 	VectorMA( point[3], frame->width - frame->origin_x, right, point[3] );
 
 	glEnableClientState( GL_VERTEX_ARRAY );
@@ -260,7 +262,7 @@ R_DrawSpriteModel(entity_t *e)
 }
 
 void
-R_DrawNullModel(void)
+R_DrawNullModel(entity_t *currententity)
 {
 	vec3_t shadelight;
 
@@ -270,7 +272,7 @@ R_DrawNullModel(void)
 	}
 	else
 	{
-		R_LightPoint(currententity->origin, shadelight);
+		R_LightPoint(currententity, currententity->origin, shadelight);
 	}
 
 	glPushMatrix();
@@ -329,7 +331,7 @@ R_DrawEntitiesOnList(void)
 	/* draw non-transparent first */
 	for (i = 0; i < r_newrefdef.num_entities; i++)
 	{
-		currententity = &r_newrefdef.entities[i];
+		entity_t *currententity = &r_newrefdef.entities[i];
 
 		if (currententity->flags & RF_TRANSLUCENT)
 		{
@@ -342,24 +344,24 @@ R_DrawEntitiesOnList(void)
 		}
 		else
 		{
-			currentmodel = currententity->model;
+			const model_t *currentmodel = currententity->model;
 
 			if (!currentmodel)
 			{
-				R_DrawNullModel();
+				R_DrawNullModel(currententity);
 				continue;
 			}
 
 			switch (currentmodel->type)
 			{
 				case mod_alias:
-					R_DrawAliasModel(currententity);
+					R_DrawAliasModel(currententity, currentmodel);
 					break;
 				case mod_brush:
-					R_DrawBrushModel(currententity);
+					R_DrawBrushModel(currententity, currentmodel);
 					break;
 				case mod_sprite:
-					R_DrawSpriteModel(currententity);
+					R_DrawSpriteModel(currententity, currentmodel);
 					break;
 				default:
 					ri.Sys_Error(ERR_DROP, "Bad modeltype");
@@ -375,7 +377,7 @@ R_DrawEntitiesOnList(void)
 
 	for (i = 0; i < r_newrefdef.num_entities; i++)
 	{
-		currententity = &r_newrefdef.entities[i];
+		entity_t *currententity = &r_newrefdef.entities[i];
 
 		if (!(currententity->flags & RF_TRANSLUCENT))
 		{
@@ -388,24 +390,24 @@ R_DrawEntitiesOnList(void)
 		}
 		else
 		{
-			currentmodel = currententity->model;
+			const model_t *currentmodel = currententity->model;
 
 			if (!currentmodel)
 			{
-				R_DrawNullModel();
+				R_DrawNullModel(currententity);
 				continue;
 			}
 
 			switch (currentmodel->type)
 			{
 				case mod_alias:
-					R_DrawAliasModel(currententity);
+					R_DrawAliasModel(currententity, currentmodel);
 					break;
 				case mod_brush:
-					R_DrawBrushModel(currententity);
+					R_DrawBrushModel(currententity, currentmodel);
 					break;
 				case mod_sprite:
-					R_DrawSpriteModel(currententity);
+					R_DrawSpriteModel(currententity, currentmodel);
 					break;
 				default:
 					ri.Sys_Error(ERR_DROP, "Bad modeltype");
@@ -427,9 +429,10 @@ R_DrawParticles2(int num_particles, const particle_t particles[],
 	float scale;
 	YQ2_ALIGNAS_TYPE(unsigned) byte color[4];
 
-	GLfloat vtx[3*num_particles*3];
-	GLfloat tex[2*num_particles*3];
-	GLfloat clr[4*num_particles*3];
+	YQ2_VLA(GLfloat, vtx, 3 * num_particles * 3);
+	YQ2_VLA(GLfloat, tex, 2 * num_particles * 3);
+	YQ2_VLA(GLfloat, clr, 4 * num_particles * 3);
+
 	unsigned int index_vtx = 0;
 	unsigned int index_tex = 0;
 	unsigned int index_clr = 0;
@@ -511,6 +514,10 @@ R_DrawParticles2(int num_particles, const particle_t particles[],
 	glColor4f(1, 1, 1, 1);
 	glDepthMask(1); /* back to normal Z buffering */
 	R_TexEnv(GL_REPLACE);
+
+	YQ2_VLAFREE(vtx);
+	YQ2_VLAFREE(tex);
+	YQ2_VLAFREE(clr);
 }
 
 void
@@ -524,9 +531,10 @@ R_DrawParticles(void)
 		int i;
 		YQ2_ALIGNAS_TYPE(unsigned) byte color[4];
 		const particle_t *p;
+ 
+		YQ2_VLA(GLfloat, vtx, 3 * r_newrefdef.num_particles);
+		YQ2_VLA(GLfloat, clr, 4*r_newrefdef.num_particles);
 
-		GLfloat vtx[3*r_newrefdef.num_particles];
-		GLfloat clr[4*r_newrefdef.num_particles];
 		unsigned int index_vtx = 0;
 		unsigned int index_clr = 0;
 
@@ -564,6 +572,9 @@ R_DrawParticles(void)
 		glColor4f( 1, 1, 1, 1 );
 		glDepthMask(GL_TRUE);
 		glEnable(GL_TEXTURE_2D);
+
+		YQ2_VLAFREE(vtx);
+		YQ2_VLAFREE(clr);
 	}
 	else
 	{
@@ -951,7 +962,7 @@ R_SetGL2D(void)
 /*
  * r_newrefdef must be set before the first call
  */
-void
+static void
 R_RenderView(refdef_t *fd)
 {
 	if ((gl_state.stereo_mode != STEREO_MODE_NONE) && gl_state.camera_separation) {
@@ -969,7 +980,7 @@ R_RenderView(refdef_t *fd)
 						// Decode the colour name from its character.
 						for (eye = 0; eye < 2; ++eye) {
 							colour = 0;
-							switch (toupper(gl1_stereo_anaglyph_colors->string[eye])) {
+							switch (toupper((unsigned char)gl1_stereo_anaglyph_colors->string[eye])) {
 								case 'B': ++colour; // 001 Blue
 								case 'G': ++colour; // 010 Green
 								case 'C': ++colour; // 011 Cyan
@@ -1072,7 +1083,7 @@ R_RenderView(refdef_t *fd)
 
 	if (!r_worldmodel && !(r_newrefdef.rdflags & RDF_NOWORLDMODEL))
 	{
-		ri.Sys_Error(ERR_DROP, "R_RenderView: NULL worldmodel");
+		ri.Sys_Error(ERR_DROP, "%s: NULL worldmodel", __func__);
 	}
 
 	if (r_speeds->value)
@@ -1149,8 +1160,8 @@ GL_GetSpecialBufferModeForStereoMode(enum stereo_modes stereo_mode) {
 	return OPENGL_SPECIAL_BUFFER_MODE_NONE;
 }
 
-void
-R_SetLightLevel(void)
+static void
+R_SetLightLevel(entity_t *currententity)
 {
 	vec3_t shadelight;
 
@@ -1160,7 +1171,7 @@ R_SetLightLevel(void)
 	}
 
 	/* save off light value for server to look at */
-	R_LightPoint(r_newrefdef.vieworg, shadelight);
+	R_LightPoint(currententity, r_newrefdef.vieworg, shadelight);
 
 	/* pick the greatest component, which should be the
 	 * same as the mono value returned by software */
@@ -1188,11 +1199,11 @@ R_SetLightLevel(void)
 	}
 }
 
-void
+static void
 RI_RenderFrame(refdef_t *fd)
 {
 	R_RenderView(fd);
-	R_SetLightLevel();
+	R_SetLightLevel (NULL);
 	R_SetGL2D();
 }
 
@@ -1223,8 +1234,8 @@ R_Register(void)
 
 	r_modulate = ri.Cvar_Get("r_modulate", "1", CVAR_ARCHIVE);
 	r_mode = ri.Cvar_Get("r_mode", "4", CVAR_ARCHIVE);
-	gl_lightmap = ri.Cvar_Get("gl_lightmap", "0", 0);
-	gl_shadows = ri.Cvar_Get("gl_shadows", "0", CVAR_ARCHIVE);
+	gl_lightmap = ri.Cvar_Get("r_lightmap", "0", 0);
+	gl_shadows = ri.Cvar_Get("r_shadows", "0", CVAR_ARCHIVE);
 	gl1_stencilshadow = ri.Cvar_Get("gl1_stencilshadow", "0", CVAR_ARCHIVE);
 	gl1_dynamic = ri.Cvar_Get("gl1_dynamic", "1", 0);
 	gl_nobind = ri.Cvar_Get("gl_nobind", "0", 0);
@@ -1239,11 +1250,12 @@ R_Register(void)
 	gl_cull = ri.Cvar_Get("gl_cull", "1", 0);
 	gl1_polyblend = ri.Cvar_Get("gl1_polyblend", "1", 0);
 	gl1_flashblend = ri.Cvar_Get("gl1_flashblend", "0", 0);
+	r_fixsurfsky = ri.Cvar_Get("r_fixsurfsky", "0", CVAR_ARCHIVE);
 
 	gl_texturemode = ri.Cvar_Get("gl_texturemode", "GL_LINEAR_MIPMAP_NEAREST", CVAR_ARCHIVE);
 	gl1_texturealphamode = ri.Cvar_Get("gl1_texturealphamode", "default", CVAR_ARCHIVE);
 	gl1_texturesolidmode = ri.Cvar_Get("gl1_texturesolidmode", "default", CVAR_ARCHIVE);
-	gl_anisotropic = ri.Cvar_Get("gl_anisotropic", "0", CVAR_ARCHIVE);
+	gl_anisotropic = ri.Cvar_Get("r_anisotropic", "0", CVAR_ARCHIVE);
 	r_lockpvs = ri.Cvar_Get("r_lockpvs", "0", 0);
 
 	gl1_palettedtexture = ri.Cvar_Get("gl1_palettedtexture", "0", CVAR_ARCHIVE);
@@ -1259,12 +1271,19 @@ R_Register(void)
 
 	r_customwidth = ri.Cvar_Get("r_customwidth", "1024", CVAR_ARCHIVE);
 	r_customheight = ri.Cvar_Get("r_customheight", "768", CVAR_ARCHIVE);
-	gl_msaa_samples = ri.Cvar_Get ( "gl_msaa_samples", "0", CVAR_ARCHIVE );
+	gl_msaa_samples = ri.Cvar_Get ( "r_msaa_samples", "0", CVAR_ARCHIVE );
 
-	gl_retexturing = ri.Cvar_Get("gl_retexturing", "1", CVAR_ARCHIVE);
+	r_retexturing = ri.Cvar_Get("r_retexturing", "1", CVAR_ARCHIVE);
+	r_scale8bittextures = ri.Cvar_Get("r_scale8bittextures", "0", CVAR_ARCHIVE);
 
 	/* don't bilerp characters and crosshairs */
-	gl_nolerp_list = ri.Cvar_Get("gl_nolerp_list", "pics/conchars.pcx pics/ch1.pcx pics/ch2.pcx pics/ch3.pcx", 0);
+	gl_nolerp_list = ri.Cvar_Get("r_nolerp_list", "pics/conchars.pcx pics/ch1.pcx pics/ch2.pcx pics/ch3.pcx", CVAR_ARCHIVE);
+	/* textures that should always be filtered, even if r_2D_unfiltered or an unfiltered gl mode is used */
+	r_lerp_list = ri.Cvar_Get("r_lerp_list", "", CVAR_ARCHIVE);
+	/* don't bilerp any 2D elements */
+	r_2D_unfiltered = ri.Cvar_Get("r_2D_unfiltered", "0", CVAR_ARCHIVE);
+	/* don't bilerp videos */
+	r_videos_unfiltered = ri.Cvar_Get("r_videos_unfiltered", "0", CVAR_ARCHIVE);
 
 	gl1_stereo = ri.Cvar_Get( "gl1_stereo", "0", CVAR_ARCHIVE );
 	gl1_stereo_separation = ri.Cvar_Get( "gl1_stereo_separation", "-0.4", CVAR_ARCHIVE );
@@ -1283,7 +1302,7 @@ R_Register(void)
 static int
 SetMode_impl(int *pwidth, int *pheight, int mode, int fullscreen)
 {
-	R_Printf(PRINT_ALL, "setting mode %d:", mode);
+	R_Printf(PRINT_ALL, "Setting mode %d:", mode);
 
 	/* mode -1 is not in the vid mode table - so we keep the values in pwidth
 	   and pheight and don't even try to look up the mode info */
@@ -1303,7 +1322,7 @@ SetMode_impl(int *pwidth, int *pheight, int mode, int fullscreen)
 		}
 	}
 
-	R_Printf(PRINT_ALL, " %d %d\n", *pwidth, *pheight);
+	R_Printf(PRINT_ALL, " %dx%d (vid_fullscreen %i)\n", *pwidth, *pheight, fullscreen);
 
 	if (!ri.GLimp_InitGraphics(fullscreen, pwidth, pheight))
 	{
@@ -1321,9 +1340,6 @@ R_SetMode(void)
 
 	fullscreen = (int)vid_fullscreen->value;
 
-	vid_fullscreen->modified = false;
-	r_mode->modified = false;
-
 	/* a bit hackish approach to enable custom resolutions:
 	   Glimp_SetMode needs these values set for mode -1 */
 	vid.width = r_customwidth->value;
@@ -1349,7 +1365,7 @@ R_SetMode(void)
 			if (gl_msaa_samples->value != 0.0f)
 			{
 				R_Printf(PRINT_ALL, "gl_msaa_samples was %d - will try again with gl_msaa_samples = 0\n", (int)gl_msaa_samples->value);
-				ri.Cvar_SetValue("gl_msaa_samples", 0.0f);
+				ri.Cvar_SetValue("r_msaa_samples", 0.0f);
 				gl_msaa_samples->modified = false;
 
 				if ((err = SetMode_impl(&vid.width, &vid.height, r_mode->value, 0)) == rserr_ok)
@@ -1379,7 +1395,7 @@ R_SetMode(void)
 }
 
 qboolean
-RI_Init()
+RI_Init(void)
 {
 	int j;
 	extern float r_turbsin[256];
@@ -1580,12 +1596,6 @@ RI_BeginFrame(float camera_separation)
 {
 	gl_state.camera_separation = camera_separation;
 
-	/* change modes if necessary */
-	if (r_mode->modified)
-	{
-		vid_fullscreen->modified = true;
-	}
-
 	// force a vid_restart if gl1_stereo has been modified.
 	if ( gl_state.stereo_mode != gl1_stereo->value ) {
 		// If we've gone from one mode to another with the same special buffer requirements there's no need to restart.
@@ -1658,21 +1668,28 @@ RI_BeginFrame(float camera_separation)
 	glEnable(GL_ALPHA_TEST);
 	glColor4f(1, 1, 1, 1);
 
-	if (gl_config.pointparameters && gl1_particle_square->modified)
+	if (gl1_particle_square->modified)
 	{
-		R_InitParticleTexture();
-
-		/* GL_POINT_SMOOTH is not implemented by some OpenGL
-		   drivers, especially the crappy Mesa3D backends like
-		   i915.so. That the points are squares and not circles
-		   is not a problem by Quake II! */
-		if (gl1_particle_square->value)
+		if (gl_config.pointparameters)
 		{
-			glDisable(GL_POINT_SMOOTH);
+			/* GL_POINT_SMOOTH is not implemented by some OpenGL
+			   drivers, especially the crappy Mesa3D backends like
+			   i915.so. That the points are squares and not circles
+			   is not a problem by Quake II! */
+			if (gl1_particle_square->value)
+			{
+				glDisable(GL_POINT_SMOOTH);
+			}
+			else
+			{
+				glEnable(GL_POINT_SMOOTH);
+			}
 		}
 		else
 		{
-			glEnable(GL_POINT_SMOOTH);
+			// particles aren't drawn as GL_POINTS, but as textured triangles
+			// => update particle texture to look square - or circle-ish
+			R_InitParticleTexture();
 		}
 
 		gl1_particle_square->modified = false;
@@ -1697,11 +1714,17 @@ RI_BeginFrame(float camera_separation)
 	}
 
 	/* texturemode stuff */
-	if (gl_texturemode->modified || (gl_config.anisotropic && gl_anisotropic->modified))
+	if (gl_texturemode->modified || (gl_config.anisotropic && gl_anisotropic->modified)
+	    || gl_nolerp_list->modified || r_lerp_list->modified
+		|| r_2D_unfiltered->modified || r_videos_unfiltered->modified)
 	{
 		R_TextureMode(gl_texturemode->string);
 		gl_texturemode->modified = false;
 		gl_anisotropic->modified = false;
+		gl_nolerp_list->modified = false;
+		r_lerp_list->modified = false;
+		r_2D_unfiltered->modified = false;
+		r_videos_unfiltered->modified = false;
 	}
 
 	if (gl1_texturealphamode->modified)
@@ -1877,6 +1900,17 @@ extern void RI_SetPalette(const unsigned char *palette);
 extern qboolean RI_IsVSyncActive(void);
 extern void RI_EndFrame(void);
 
+/*
+=====================
+RI_EndWorldRenderpass
+=====================
+*/
+static qboolean
+RI_EndWorldRenderpass( void )
+{
+	return true;
+}
+
 Q2_DLL_EXPORTED refexport_t
 GetRefAPI(refimport_t imp)
 {
@@ -1917,8 +1951,13 @@ GetRefAPI(refimport_t imp)
 
 	re.SetPalette = RI_SetPalette;
 	re.BeginFrame = RI_BeginFrame;
+	re.EndWorldRenderpass = RI_EndWorldRenderpass;
 	re.EndFrame = RI_EndFrame;
 
+    // Tell the client that we're unsing the
+	// new renderer restart API.
+    ri.Vid_RequestRestart(RESTART_NO);
+
 	return re;
 }
 
