@@ -34,13 +34,26 @@
   // using system headers for their parsers/indexers but glad for real build
   // (in glad glFoo is just a #define to glad_glFoo or sth, which screws up autocompletion)
   // (you may have to configure your IDE to #define IN_IDE_PARSER, but not for building!)
+#ifdef YQ2_GL3_GLES3
+  #include <GLES3/gl32.h>
+#else // desktop GL3
   #define GL_GLEXT_PROTOTYPES 1
   #include <GL/gl.h>
   #include <GL/glext.h>
+#endif
+
 #else
+
+#ifdef YQ2_GL3_GLES3
+  #include "../glad-gles3/include/glad/glad.h"
+  // yes, this is a bit hacky, but it works :-P
+  #define glDepthRange glDepthRangef
+#else // desktop GL3
   #include "../glad/include/glad/glad.h"
 #endif
 
+#endif
+
 #include "../../ref_shared.h"
 
 #include "HandmadeMath.h"
@@ -81,10 +94,11 @@ enum {
 	GL3_ATTRIB_LIGHTFLAGS = 5  // uint, each set bit means "dyn light i affects this surface"
 };
 
-// TODO: do we need the following configurable?
-static const int gl3_solid_format = GL_RGB;
+// always using RGBA now, GLES3 on RPi4 doesn't work otherwise
+// and I think all modern GPUs prefer 4byte pixels over 3bytes
+static const int gl3_solid_format = GL_RGBA;
 static const int gl3_alpha_format = GL_RGBA;
-static const int gl3_tex_solid_format = GL_RGB;
+static const int gl3_tex_solid_format = GL_RGBA;
 static const int gl3_tex_alpha_format = GL_RGBA;
 
 extern unsigned gl3_rawpalette[256];
@@ -116,7 +130,8 @@ typedef struct
 typedef struct
 {
 	GLuint shaderProgram;
-	GLint uniLmScales;
+	GLint uniVblend;
+	GLint uniLmScalesOrTime; // for 3D it's lmScales, for 2D underwater PP it's time
 	hmm_vec4 lmScales[4];
 } gl3ShaderInfo_t;
 
@@ -140,8 +155,7 @@ typedef struct
 
 typedef struct
 {
-	hmm_mat4 transProjMat4;
-	hmm_mat4 transViewMat4;
+	hmm_mat4 transProjViewMat4; // gl3state.projMat3D * gl3state.viewMat3D - so we don't have to do this in the shader
 	hmm_mat4 transModelMat4;
 
 	GLfloat scroll; // for SURF_FLOWING
@@ -196,6 +210,13 @@ typedef struct
 	int currentlightmap; // lightmap_textureIDs[currentlightmap] bound to GL_TEXTURE1
 	GLuint currenttmu; // GL_TEXTURE0 or GL_TEXTURE1
 
+	// FBO for postprocess effects (like under-water-warping)
+	GLuint ppFBO;
+	GLuint ppFBtex; // ppFBO's texture for color buffer
+	int ppFBtexWidth, ppFBtexHeight;
+	GLuint ppFBrbo; // ppFBO's renderbuffer object for depth and stencil buffer
+	qboolean ppFBObound; // is it currently bound (rendered to)?
+
 	//float camera_separation;
 	//enum stereo_modes stereo_mode;
 
@@ -208,6 +229,9 @@ typedef struct
 	// NOTE: make sure si2D is always the first shaderInfo (or adapt GL3_ShutdownShaders())
 	gl3ShaderInfo_t si2D;      // shader for rendering 2D with textures
 	gl3ShaderInfo_t si2Dcolor; // shader for rendering 2D with flat colors
+	gl3ShaderInfo_t si2DpostProcess; // shader to render postprocess FBO, when *not* underwater
+	gl3ShaderInfo_t si2DpostProcessWater; // shader to apply water-warp postprocess effect
+
 	gl3ShaderInfo_t si3Dlm;        // a regular opaque face (e.g. from brush) with lightmap
 	// TODO: lm-only variants for gl_lightmap 1
 	gl3ShaderInfo_t si3Dtrans;     // transparent is always w/o lightmap
@@ -244,6 +268,8 @@ typedef struct
 	GLuint uni3DUBO;
 	GLuint uniLightsUBO;
 
+	hmm_mat4 projMat3D;
+	hmm_mat4 viewMat3D;
 } gl3state_t;
 
 extern gl3config_t gl3config;
@@ -393,6 +419,7 @@ extern void GL3_Draw_PicScaled(int x, int y, char *pic, float factor);
 extern void GL3_Draw_StretchPic(int x, int y, int w, int h, char *pic);
 extern void GL3_Draw_CharScaled(int x, int y, int num, float scale);
 extern void GL3_Draw_TileClear(int x, int y, int w, int h, char *pic);
+extern void GL3_DrawFrameBufferObject(int x, int y, int w, int h, GLuint fboTexture, const float v_blend[4]);
 extern void GL3_Draw_Fill(int x, int y, int w, int h, int c);
 extern void GL3_Draw_FadeScreen(void);
 extern void GL3_Draw_Flash(const float color[4], float x, float y, float w, float h);
@@ -488,7 +515,9 @@ extern cvar_t *r_customwidth;
 extern cvar_t *r_customheight;
 
 extern cvar_t *r_2D_unfiltered;
+extern cvar_t *r_videos_unfiltered;
 extern cvar_t *gl_nolerp_list;
+extern cvar_t *r_lerp_list;
 extern cvar_t *gl_nobind;
 extern cvar_t *r_lockpvs;
 extern cvar_t *r_novis;
@@ -513,6 +542,7 @@ extern cvar_t *r_lightlevel;
 extern cvar_t *gl3_overbrightbits;
 extern cvar_t *gl3_particle_fade_factor;
 extern cvar_t *gl3_particle_square;
+extern cvar_t *gl3_colorlight;
 
 extern cvar_t *r_modulate;
 extern cvar_t *gl_lightmap;
