@@ -223,41 +223,6 @@ Mod_ForName (char *name, model_t *parent_model, qboolean crash)
 	return mod;
 }
 
-
-/*
-===============
-Mod_PointInLeaf
-===============
-*/
-mleaf_t *
-Mod_PointInLeaf (vec3_t p, const model_t *model)
-{
-	mnode_t		*node;
-
-	if (!model || !model->nodes)
-	{
-		ri.Sys_Error(ERR_DROP, "%s: bad model", __func__);
-		return NULL;
-	}
-
-	node = model->nodes;
-	while (node->contents == -1)
-	{
-		float d;
-		cplane_t *plane;
-
-		plane = node->plane;
-		d = DotProduct (p,plane->normal) - plane->dist;
-		if (d > 0)
-			node = node->children[0];
-		else
-			node = node->children[1];
-	}
-
-	return (mleaf_t *)node;
-}
-
-
 /*
 ==============
 Mod_ClusterPVS
@@ -292,26 +257,17 @@ by taking the brightest component
 static void
 Mod_LoadLighting (model_t *loadmodel, byte *mod_base, lump_t *l)
 {
-	int		i, size;
-	byte	*in;
+	int	size;
 
 	if (!l->filelen)
 	{
 		loadmodel->lightdata = NULL;
 		return;
 	}
-	size = l->filelen/3;
+
+	size = l->filelen;
 	loadmodel->lightdata = Hunk_Alloc(size);
-	in = (void *)(mod_base + l->fileofs);
-	for (i=0 ; i<size ; i++, in+=3)
-	{
-		if (in[0] > in[1] && in[0] > in[2])
-			loadmodel->lightdata[i] = in[0];
-		else if (in[1] > in[0] && in[1] > in[2])
-			loadmodel->lightdata[i] = in[1];
-		else
-			loadmodel->lightdata[i] = in[2];
-	}
+	memcpy(loadmodel->lightdata, mod_base + l->fileofs, size);
 }
 
 
@@ -699,9 +655,13 @@ Mod_LoadFaces (model_t *loadmodel, byte *mod_base, lump_t *l)
 			out->styles[i] = in->styles[i];
 		i = LittleLong(in->lightofs);
 		if (i == -1)
+		{
 			out->samples = NULL;
+		}
 		else
-			out->samples = loadmodel->lightdata + i/3;
+		{
+			out->samples = loadmodel->lightdata + i;
+		}
 
 		// set the drawing flags flag
 
@@ -1043,10 +1003,12 @@ Mod_LoadBrushModel(model_t *mod, void *buffer, int modfilelen)
 	if(surfEdgeCount < MAX_MAP_SURFEDGES) // else it errors out later anyway
 		hunkSize += calcLumpHunkSize(&header->lumps[LUMP_SURFEDGES], sizeof(int), sizeof(int), 24);
 
-	// lighting is a special case, because we keep only 1 byte out of 3 (=> no colored lighting in soft renderer)
+	// lighting is a special case, because we keep only 1 byte out of 3
+	// (=> no colored lighting in soft renderer by default)
 	{
-		int size = header->lumps[LUMP_LIGHTING].filelen/3;
+		int size = header->lumps[LUMP_LIGHTING].filelen;
 		size = (size + 31) & ~31;
+		/* save color data */
 		hunkSize += size;
 	}
 
