@@ -32,7 +32,7 @@
 #include "shared.h"
 #include "crc.h"
 
-#define YQ2VERSION "8.01"
+#define YQ2VERSION "8.10"
 #define BASEDIRNAME "baseq2"
 
 #ifndef YQ2OSTYPE
@@ -50,7 +50,11 @@
 #ifdef _WIN32
  #define CFGDIR "YamagiQ2"
 #else
- #define CFGDIR ".yq2"
+ #ifndef __HAIKU__
+   #define CFGDIR ".yq2"
+ #else
+   #define CFGDIR "yq2"
+ #endif
 #endif
 
 /* ================================================================== */
@@ -711,11 +715,11 @@ void FS_CreatePath(char *path);
 
 void Com_BeginRedirect(int target, char *buffer, int buffersize, void (*flush)(int, char *));
 void Com_EndRedirect(void);
-void Com_Printf(char *fmt, ...) __attribute__ ((format (printf, 1, 2)));
-void Com_DPrintf(char *fmt, ...) __attribute__ ((format (printf, 1, 2)));
+void Com_Printf(char *fmt, ...) PRINTF_ATTR(1, 2);
+void Com_DPrintf(char *fmt, ...) PRINTF_ATTR(1, 2);
 void Com_VPrintf(int print_level, const char *fmt, va_list argptr); /* print_level is PRINT_ALL or PRINT_DEVELOPER */
-void Com_MDPrintf(char *fmt, ...) __attribute__ ((format (printf, 1, 2)));
-YQ2_ATTR_NORETURN void Com_Error(int code, char *fmt, ...) __attribute__ ((format (printf, 2, 3)));
+void Com_MDPrintf(char *fmt, ...) PRINTF_ATTR(1, 2);
+YQ2_ATTR_NORETURN void Com_Error(int code, char *fmt, ...) PRINTF_ATTR(2, 3);
 YQ2_ATTR_NORETURN void Com_Quit(void);
 
 /* Ugly work around for unsupported
