@@ -242,13 +242,8 @@ G_UseTargets(edict_t *ent, edict_t *activator)
 		return;
 	}
 
-	if (!activator)
-	{
-		return;
-	}
-
 	/* print the message */
-	if ((ent->message) && !(activator->svflags & SVF_MONSTER))
+	if (activator && (ent->message) && !(activator->svflags & SVF_MONSTER))
 	{
 		gi.centerprintf(activator, "%s", ent->message);
 
